@@ -1,9 +1,11 @@
 """Utility script to help manage Development with various versions of Python
 in conjunction with Virtual Environments and the pyenv module.
 """
+
 # Core Library modules
 import ast
 import configparser
+import fnmatch
 import os
 import re
 import shutil
@@ -16,6 +18,7 @@ from typing import Any
 
 # Third party modules
 import click
+import magic
 
 _PLATFORM = sys.platform
 if _PLATFORM != "win32":
@@ -576,15 +579,22 @@ def pyenv(ctx: Any, use_pyenv: str, use_force: str) -> None:
 
 
 @cli.command()
+@click.option("-n", "--noall", "create_all", flag_value="n")
+@click.option("-a", "--all", "create_all", flag_value="y", default=True)
 @click.argument("name")
 @click.argument("project_name")
-def mkproject(name: str, project_name: str) -> None:
+def mkproject(create_all: str, name: str, project_name: str) -> None:
     """Creates a Virtual Environment from a specific Python version.
+    \b
+    Options:
+        -n, --noall:      Only create the virtual environment. Skips project area.
 
     \b
     Args:
         name:           The Pynball friendly version name.
         project_name:   The project name only. Not the path.
+        \f
+        create_all:     Determines if the project folder gets created.
     """
     if _WORKON_HOME == Path("") or _PROJECT_HOME == Path(""):
         message = """Virtualenv-wrapper is not configured on your system:
@@ -604,29 +614,34 @@ def mkproject(name: str, project_name: str) -> None:
         _feedback(message, "warning")
         return
     for directory in [_WORKON_HOME, _PROJECT_HOME]:
-        new_path = directory / project_name
-        try:
-            new_path.mkdir(parents=False, exist_ok=False)
-            if directory == _WORKON_HOME:
-                python_path = version_path / "python.exe"
-                _execute(
-                    "virtualenv",
-                    f"-p={str(python_path)}",
-                    str(new_path),
+        if directory == _PROJECT_HOME and create_all == "n":
+            continue
+        else:
+            new_path = directory / project_name
+            try:
+                new_path.mkdir(parents=False, exist_ok=False)
+                if directory == _WORKON_HOME:
+                    python_path = version_path / "python.exe"
+                    _execute(
+                        "virtualenv",
+                        f"-p={str(python_path)}",
+                        str(new_path),
+                    )
+                    (new_path / ".project").touch()
+                    (new_path / ".project").write_text(
+                        f"{_PROJECT_HOME / project_name}"
+                    )
+            except FileNotFoundError:
+                message = (
+                    f"Project: '{project_name}' has NOT be created - "
+                    f"{directory} does not exist"
                 )
-                (new_path / ".project").touch()
-                (new_path / ".project").write_text(f"{_PROJECT_HOME / project_name}")
-        except FileNotFoundError:
-            message = (
-                f"Project: '{project_name}' has NOT be created - "
-                f"{directory} does not exist"
-            )
-            _feedback(message, "warning")
-            return
-        except FileExistsError:
-            message = f"The directory '{project_name}' already exits"
-            _feedback(message, "warning")
-            return
+                _feedback(message, "warning")
+                return
+            except FileExistsError:
+                message = f"The directory '{project_name}' already exits"
+                _feedback(message, "warning")
+                return
 
 
 @cli.command()
@@ -706,34 +721,103 @@ def lsproject() -> None:
 
 
 @cli.command()
+@click.pass_context
 @click.argument("old_name")
 @click.argument("new_name")
-def mvproject(old_name: str, new_name: str) -> None:
-    """Renames a virtual Environment"""
-    replace_text_files = [
-        _WORKON_HOME / old_name / "Scripts" / "activate",
-        _WORKON_HOME / old_name / "Scripts" / "activate.bat",
-        _WORKON_HOME / old_name / "Scripts" / "activate.fish",
-        _WORKON_HOME / old_name / "Scripts" / "activate.nu",
-        _WORKON_HOME / old_name / "Scripts" / "activate.ps1",
-        _WORKON_HOME / old_name / ".project",
-        _WORKON_HOME / old_name / "pyvenv.cfg",
+def mvproject(ctx: Any, old_name: str, new_name: str) -> None:
+    """Renames an existing project.
+
+    \b
+    Args:
+        old_name:   The existing project to rename.
+        new_name:   The new project name.
+    """
+
+    project_root = _PROJECT_HOME / old_name
+    virt_project_root = _WORKON_HOME / old_name
+    files = project_root.rglob("*")
+    mime = magic.Magic(mime=True)
+    dir_name_change, file_search, file_name_change = [], [], []
+    dir_name_change.append(project_root)
+    ignore_dirs = [
+        ".idea",
     ]
-    rename_files = [_WORKON_HOME / old_name, _PROJECT_HOME / old_name]
+    ignore_files = [
+        "filetype.py",
+        "hello",
+        "search*.py",
+    ]
+
+    def glob_to_re(glob_list: list) -> str:
+        re_list = [fnmatch.translate(x) for x in glob_list]
+        combined = "(" + ")|(".join(re_list) + ")"
+        return combined
+
+    ignore_files_re = glob_to_re(ignore_files)
+    ignore_dirs_set = set(ignore_dirs)
+
+    for file in files:
+        if file.is_file():
+            if (
+                "text" in mime.from_file(str(file))
+                and not re.match(ignore_files_re, file.name)
+                and not set(file.parent.parts).intersection(ignore_dirs_set)
+            ):
+                file_search.append(file)
+            if old_name in str(file):
+                file_name_change.append(file)
+        elif old_name in str(file) and file.stem not in ignore_dirs:
+            dir_name_change.append(file)
+
+    # replace text inside files
+    for file in file_search:
+        text = file.read_text()
+        text = text.replace(old_name, new_name)
+        file.write_text(text)
+
+    # replace text in filenames
+    for file in file_name_change:
+        old_file_name = file.name
+        new_file_name = old_file_name.replace(old_name, new_name)
+        file.rename(file.parent / new_file_name)
+
+    # replace text in folder names
+    for directory in reversed(dir_name_change):
+        old_dir_name = directory.stem
+        new_dir_name = old_dir_name.replace(old_name, new_name)
+        directory.rename(directory.parent / new_dir_name)
+
+    # find version of python for old virtual environment
+    pattern1 = r"(?<=version_info = )\d{1,2}.\d{1,2}.\d{1,2}"
+    pattern2 = r"(?<=version = )\d{1,2}.\d{1,2}.\d{1,2}"
+    envcfg = virt_project_root / "pyvenv.cfg"
+    if envcfg.exists():
+        cfg_content = envcfg.read_text()
+        for pattern in [pattern1, pattern2]:
+            try:
+                virtver = re.search(pattern, cfg_content).group(0)  # type: ignore
+                break
+            except AttributeError:
+                message = "Cannot ascertain existing python version"
+                _feedback(message, "warning")
+                exit(1)
+    else:
+        message = "Cannot ascertain existing python version"
+        _feedback(message, "warning")
+        exit(1)
+
+    # delete existing virtual environment
     try:
-        for file in rename_files:
-            newname = file.parent / f"{new_name}{file.suffix}"
-            file.rename(newname)
-        for file in replace_text_files:
-            _file_word_replace(file, old_name, new_name)
+        shutil.rmtree(virt_project_root, onerror=del_rw)
     except FileNotFoundError:
-        message = f"Virtual environment '{old_name}' does not exist"
+        message = f"Project: '{old_name}'does not exist"
         _feedback(message, "warning")
-        return
-    except FileExistsError:
-        message = f"Virtual environment '{new_name}' already exits"
+    except PermissionError:
+        message = f"Insufficient permissions to delete {old_name}"
         _feedback(message, "warning")
-        return
+
+    # Create the new virtual environment
+    ctx.invoke(mkproject, create_all="n", name=virtver, project_name=new_name)
 
 
 @cli.command()
